//=============================================================================
//  Viool plugin / adjusted by Serge tkint for violin
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License version 2.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//=============================================================================

import QtQuick 2.0
import MuseScore 3.0

MuseScore {
   menuPath: "Plugins.Violin color"
   version: "3.0"
   description: qsTr("This plugin names notes for violin")

   property variant colors : [
            "#ffcf0f", "#ffcf0f", "#ffcf0f", "#ffcf0f", "#ffcf0f", "#ffcf0f", "#ffcf0f",
            "#00ff00", "#00ff00", "#00ff00", "#00ff00", "#00ff00", "#00ff00", "#00ff00",
            "#0000ff", "#0000ff", "#0000ff", "#0000ff", "#0000ff", "#0000ff", "#0000ff", 
            "#ff0000", "#ff0000", "#ff0000", "#ff0000", "#ff0000", "#ff0000", "#ff0000", 
            "#000000", "#000000", "#000000", "#000000", "#000000", "#000000", "#000000"
            ]

   property variant fingerings : [ 
            "0", "1", "1", "2", "2", "3", "3", 
            "0", "1", "1", "2", "2", "3", "3", 
            "0", "1", "1", "2", "2", "3", "3", 
            "0", "1", "1", "2", "2", "3", "3", "4" 
            ]

   property variant black : "#000000"
      
   function nameChord (notes, text) {
      for (var i = 0; i < notes.length; i++) {
         var sep = ","; // change to "\n" if you want them vertically
         if ( i > 0 )
            text.text = sep + text.text;

         if (typeof notes[i].tpc === "undefined") // just in case
            return
    
         var index = notes[i].pitch - 55;
         if(index >= 0 && index < fingerings.length){ 
            text.text = fingerings[index];
            text.color = colors[index];
         }

//	do you play the 4fingers or the 0fingers?
//         if(index == 7){ 
//            text.text = "4";
//            text.color = colors[index-2];
//         }

      } // end for note
   }

   function colorNote(note) {
      if (note.color == black)
        note.color = colors[note.pitch % 12];
      else
        note.color = black;

      if (note.accidental) {
         if (note.accidental.color == black)
           note.accidental.color = colors[note.pitch % 12];
         else
            note.accidental.color = black;
         }

       for (var i = 0; i < note.dots.length; i++) {
         if (note.dots[i]) {
            if (note.dots[i].color == black)
               note.dots[i].color = colors[note.pitch % 12];
            else
               note.dots[i].color = black;
            }
         }
       }

   onRun: {
      if (typeof curScore === 'undefined')
        Qt.quit();
      var cursor = curScore.newCursor();
      var startStaff;
      var endStaff;
      var endTick;
      var fullScore = false;
      cursor.rewind(1);
      if (!cursor.segment) { // no selection
        fullScore = true;
        startStaff = 0; // start with 1st staff
        endStaff  = curScore.nstaves - 1; // and end with last
      } else {
        startStaff = cursor.staffIdx;
        cursor.rewind(2);
        if (cursor.tick == 0) {
          // this happens when the selection includes the last measure of the score.
          endTick = curScore.lastSegment.tick + 1;
        } else {
          endTick = cursor.tick;
        }
        endStaff   = cursor.staffIdx;
      }
      console.log(startStaff + " - " + endStaff + " - " + endTick)

      for (var staff = startStaff; staff <= endStaff; staff++) {
        for (var voice = 0; voice < 4; voice++) {
          cursor.rewind(1); // beginning of selection
          cursor.voice    = voice;
          cursor.staffIdx = staff;

          if (fullScore)  // no selection
            cursor.rewind(0); // beginning of score

          while (cursor.segment && (fullScore || cursor.tick < endTick)) {
            if (cursor.element && cursor.element.type == Element.CHORD) {
              var text = newElement(Element.STAFF_TEXT);

              var graceChords = cursor.element.graceNotes;
              for (var i = 0; i < graceChords.length; i++) {
                // iterate through all grace chords
                var notes = graceChords[i].notes;
                nameChord(notes, text);
                // there seems to be no way of knowing the exact horizontal pos.
                // of a grace note, so we have to guess:
                text.offsetX = -2.5 * (graceChords.length - i);
                switch (voice) {
                  case 0: text.offsetY =  1; break;
                  case 1: text.offsetY = 10; break;
                  case 2: text.offsetY = -1; break;
                  case 3: text.offsetY = 12; break;
                }

                cursor.add(text);
                // new text for next element
                text  = newElement(Element.STAFF_TEXT);
              }

              var notes = cursor.element.notes;
              nameChord(notes, text);

              switch (voice) {
                case 0: text.offsetY =  1; break;
                case 1: text.offsetY = 10; break;
                case 2: text.offsetY = -1; break;
                case 3: text.offsetY = 12; break;
              }
              if ((voice == 0) && (notes[0].pitch > 83))
                text.offsetX = 1;
              cursor.add(text);
            } // end if CHORD
            cursor.next();
          } // end while segment
        } // end for voice
      } // end for staff
      Qt.quit();
   } // end onRun
   
   
}
